@@ -1,8 +1,8 @@
 # -*- coding: utf-8 -*-
 
 ###
-### $Release: 0.5.0 $
-### $Copyright: copyright(c) 2010 kuwata-lab.com all rights reserved $
+### $Release: 0.6.0 $
+### $Copyright: copyright(c) 2010-2011 kuwata-lab.com all rights reserved $
 ### $License: MIT License $
 ###
 
@@ -19,46 +19,91 @@ import os, re
 from glob import glob
 from kook.utils import read_file, write_file
 
-release   = prop('release', '0.5.0')
+release   = prop('release', '0.6.0')
 package   = prop('package', 'Oktest')
-copyright = prop('copyright', "copyright(c) 2010 kuwata-lab.com all rights reserved")
+copyright = prop('copyright', "copyright(c) 2010-2011 kuwata-lab.com all rights reserved")
 license   = "MIT License"
 #kook_default_product = 'test'
 
 python = prop('python', 'python')
 
-
 @recipe
-def task_test(c):
-    system(c%"$(python) test/oktest_test.py")
+@spices("-a: do with python 2.4, 2.5, 2.6, 2.7, 3.0, 3.1, 3.2a2")
+def task_test(c, *args, **kwargs):
+    if kwargs.get('a', None):
+        _do_for_all_version(c, "PYTHON=$(bin) $(bin) test/oktest_test.py")
+    else:
+        system(c%"$(python) test/oktest_test.py")
 
-@recipe
-def task_test_all(c):
-    "do test with python 2.5, 2.6, 2.7, 3.0, 3.1"
+def _do_for_all_version(c, command):
     versions = [
+        ('2.4', '/opt/local/bin/python2.4'),
         ('2.5', '/opt/local/bin/python2.5'),
         ('2.6', '/opt/local/bin/python2.6'),
         ('2.7', '/opt/local/bin/python2.7'),
         ('3.0', '/usr/local/python/3.0.1/bin/python'),
         ('3.1', '/usr/local/python/3.1/bin/python'),
+        ('3.2', '/usr/local/python/3.2a2/bin/python'),
     ]
     for ver, bin in versions:
         print("#")
         print("# python " + ver)
         print("#")
-        system(c%"PYTHON=$(bin) $(bin) test/oktest_test.py")
+        fpath = "test/oktest_test.py"
+        try:
+            if ver == '2.4':
+                mv(fpath, fpath+'.bkup')
+                cp(fpath+'.bkup', fpath)
+                s = open(fpath+'.bkup').read()
+                line = 'from __future__ import with_statement'
+                open(fpath, 'w').write(s.replace(line, '#' + line))
+            system(c%command)
+        finally:
+            if ver == '2.4':
+                mv(fpath+'.bkup', fpath)
+
+@recipe
+@ingreds('test/doc_test.py')
+@spices("-a: do with python 2.4, 2.5, 2.6, 2.7, 3.0, 3.1, 3.2a2")
+def task_doc_test(c, *args, **kwargs):
+    """invoke 'test/doc_test.py'"""
+    if kwargs.get('a', None):
+        _do_for_all_version(c, '$(python) ' + c.ingred)
+    else:
+        system(c%'python $(ingred)')
+
+@recipe
+@product('test/doc_test.py')
+@ingreds('README.txt')
+def file_doc_test_py(c):
+    cont = open(c.ingred).read()
+    pat = r'\n(Tracer\n=+\n.*?)\nHelpers Reference'
+    m = re.compile(pat, re.S).search(cont)
+    s = m.group(1)
+    s = re.compile(r'^(\S)', re.M).sub(r'#\1', s)
+    #
+    f = open(c.product, 'w')
+    f.write(
+        "import re\n"
+        "from oktest import ok\n"
+        "\n"
+        "if True:\n"
+        "\n"
+    )
+    f.write(s)
+    f.close()
 
 @recipe
 def task_edit(c):
-    """update 0.5.0, $Copyrigh$, and MIT License in files"""
+    """update 0.6.0, $Copyrigh$, and MIT License in files"""
     def replacer(s):
         s = re.sub(r'\$Release:[^%]*?\$',    '$Release: %s $'   % release,   s)
         s = re.sub(r'\$Copyright:[^%]*?\$',  '$Copyright: %s $' % copyright, s)
         s = re.sub(r'\$License:[^%]*?\$',    '$License: %s $'   % license,   s)
+        s = re.sub(r'%s-\d+\.\d+\.\d+' % package, '%s-%s' % (package, release), s)
         return s
     filenames = read_file('MANIFEST').splitlines()
     filenames.remove('Kookbook.py')
-    filenames.append('README.wiki')
     edit(filenames, by=replacer)
 
 
@@ -146,6 +191,7 @@ def file_website_index_html(c):
         s = s.replace('$Release:[^%]*?$', '$Release: %s $' % release)
         s = s.replace('$'+'Release'+'$', release)
         s = s.replace('README', 'Oktest - a new style testing library -')
+        s = s.replace('See CHANGES.txt', 'See <a href="CHANGES.txt">CHANGES.txt</a>')
         #s = re.sub(r'^<h(\d)>(.*?)</h\d>', r, s)
         pat = re.compile(r'^<h(\d)>(.*?)</h\d>', re.M)
         def r(m):
@@ -155,10 +201,21 @@ def file_website_index_html(c):
         s = re.sub(pat, r, s)
         return s
     edit(c.product, by=f)
-    with chdir("website"):
-        system("zip -r ../Oktest.zip index.html style.css")
 
 @recipe
-@ingreds('website/index.html')
+@product('website/CHANGES.txt')
+@ingreds('CHANGES.txt')
+def file_CHANGES_txt(c):
+    cp(c.ingred, c.product)
+
+@recipe
+@product('Oktest.zip')
+@ingreds('website/index.html', 'website/CHANGES.txt')
+def file_Oktest_zip(c):
+    with chdir("website"):
+        system("zip -r ../Oktest.zip index.html style.css CHANGES.txt")
+
+@recipe
+@ingreds('Oktest.zip')
 def task_website(c):
     pass
