@@ -1,6 +1,6 @@
 ###
-### $Release: 0.5.0 $
-### $Copyright: copyright(c) 2010 kuwata-lab.com all rights reserved $
+### $Release: 0.6.0 $
+### $Copyright: copyright(c) 2010-2011 kuwata-lab.com all rights reserved $
 ### $License: MIT License $
 ###
 from __future__ import with_statement
@@ -27,7 +27,7 @@ import oktest
 echo = sys.stdout.write
 python_command = os.environ.get('PYTHON', 'python')
 
-def do_test_with(desc, script, expected,  _pat=re.compile(r'0\.00[01]s')):
+def do_test_with(desc, script, expected,  _pat=re.compile(r'0\.00[\d]s')):
     filename = '_test_.py'
     try:
         echo("- %s..." % desc)
@@ -38,7 +38,7 @@ def do_test_with(desc, script, expected,  _pat=re.compile(r'0\.00[01]s')):
         #output = sout.read();  sout.close()
         #output_err = serr.read();  serr.close()
         if isinstance(output, str):
-            output = re.sub(r' at 0x[0-9a-f]{6}', '', output)
+            output = re.sub(r' at 0x[0-9a-f]{6,9}', '', output)
         if output == expected:
             echo("done.\n")
         elif _pat.sub('', output) == _pat.sub('', expected):
@@ -48,10 +48,14 @@ def do_test_with(desc, script, expected,  _pat=re.compile(r'0\.00[01]s')):
         else:
             echo("FAILED.\n")
             if (isinstance(output, str) and isinstance(expected, str)):
-                lf = (not output or output[-1] != "\n") and "\n" or ""
-                echo("output: %s%s" % (output, lf))
-                lf = (not expected or expected[-1] != "\n") and "\n" or ""
-                echo("expected: %s%s" % (expected, lf))
+                import difflib
+                for x in difflib.unified_diff(expected.splitlines(True), output.splitlines(True), 'expected', 'actual', n=2):
+                    echo(x)
+                    #echo(repr(x) + "\n")
+                #lf = (not output or output[-1] != "\n") and "\n" or ""
+                #echo("output: %s%s" % (output, lf))
+                #lf = (not expected or expected[-1] != "\n") and "\n" or ""
+                #echo("expected: %s%s" % (expected, lf))
             else:
                 echo('%s != %s' % (repr(output), repr(expected)))
     finally:
@@ -99,8 +103,8 @@ class BarTest(object):
 run('.*Test$')
 """[1:]
 expected = r"""
-* BarTest.test_minus ... [ok]
 * FooTest.test_plus ... [ok]
+* BarTest.test_minus ... [ok]
 """[1:]
 do_test_with(desc, script, expected)
 
@@ -150,6 +154,8 @@ test_3() called.
 after() called.
 after_all() called.
 """[1:]
+if python2 and sys.version_info[1] <= 4:
+    expected = expected.replace("int() with base 10: 'abc'", 'int(): abc')
 do_test_with(desc, script, expected)
 
 ###
@@ -189,6 +195,8 @@ test_3() called.
   - _test_.py:16:  int('abc')
 tearDown() called.
 """[1:]
+if python2 and sys.version_info[1] <= 4:
+    expected = expected.replace("int() with base 10: 'abc'", 'int(): abc')
 do_test_with(desc, script, expected)
 
 
@@ -672,9 +680,35 @@ class BazTestCase(object):
 run()
 """
 expected = r"""
+* FooTest.test_1 ... [ok]
 * BarTestCase.test_2 ... [ok]
 * BazTestCase.test_3 ... [ok]
-* FooTest.test_1 ... [ok]
+"""[1:]
+do_test_with(desc, script, expected)
+
+## _min_firstlineno_of_methods
+desc = "_min_firstlineno_of_methods"
+script = r"""
+import oktest                  # 1
+class FooTest(object):         # 2
+    def test_2(self):          # 3
+        pass                   # 4
+    def test_1(self):          # 5
+        pass                   # 6
+class BarTest(FooTest):        # 7
+    def test_1(self):          # 8
+        pass                   # 9
+class BazTest(FooTest):        # 10
+    def _test_1(self):         # 11
+        pass                   # 12
+print(oktest._min_firstlineno_of_methods(FooTest))   #=> 3
+print(oktest._min_firstlineno_of_methods(BarTest))   #=> 8
+print(oktest._min_firstlineno_of_methods(BazTest))   #=> -1
+"""[1:]
+expected = r"""
+3
+8
+-1
 """[1:]
 do_test_with(desc, script, expected)
 
@@ -685,6 +719,7 @@ desc = "dummy_file (with with-statement)"
 script = r"""
 from __future__ import with_statement
 from oktest import *
+from oktest.helper import *
 class FooTest(object):
     def test_dummy_file(self):
         with dummy_file('_dummy_.txt', 'hogehoge') as f:
@@ -702,6 +737,7 @@ if with_statement_supported:
 desc = "dummy_file (without with-statement)"
 script = r"""
 from oktest import *
+from oktest.helper import *
 class FooTest(object):
     def test_dummy_file2(self):
         filename = '_dummy_.txt'
@@ -720,6 +756,7 @@ desc = "dummy_dir (with with-statement)"
 script = r"""
 from __future__ import with_statement
 from oktest import *
+from oktest.helper import *
 class FooTest(object):
     def test_dummy_dir(self):
         with dummy_dir('_dummy_.d') as d:
@@ -736,6 +773,7 @@ if with_statement_supported:
 desc = "dummy_dir (without with-statement)"
 script = r"""
 from oktest import *
+from oktest.helper import *
 class FooTest(object):
     def test_dummy_dir2(self):
         dirname = '_dummy_.d'
@@ -754,6 +792,7 @@ script = r"""
 from __future__ import with_statement
 import os
 from oktest import *
+from oktest.helper import *
 class FooTest(object):
     def test_dummy_environ_vars(self):
         os.environ['AAAA'] = 'hoge'
@@ -777,6 +816,7 @@ desc = "dummy_environ_vars (without with-statement)"
 script = r"""
 import os
 from oktest import *
+from oktest.helper import *
 class FooTest(object):
     def test_dummy_environ_vars2(self):
         os.environ['AAAA'] = 'hoge'
@@ -801,6 +841,7 @@ script = r"""
 from __future__ import with_statement
 import os
 from oktest import *
+from oktest.helper import *
 class FooTest(object):
     def test_dummy_values(self):
         d = {'A': 10, 'B': 20, 'C': 30 }
@@ -818,6 +859,7 @@ desc = "dummy_values (without with-statement)"
 script = r"""
 import os
 from oktest import *
+from oktest.helper import *
 class FooTest(object):
     def test_dummy_values2(self):
         d = {'A': 10, 'B': 20, 'C': 30 }
@@ -836,6 +878,7 @@ script = r"""
 from __future__ import with_statement
 import os
 from oktest import *
+from oktest.helper import *
 class Foo(object):
   def __init__(self, x, y):
     self.x = x
@@ -862,6 +905,7 @@ desc = "dummy_attrs (without with-statement)"
 script = r"""
 import os
 from oktest import *
+from oktest.helper import *
 class Foo(object):
   def __init__(self, x, y):
     self.x = x
@@ -883,12 +927,69 @@ run(FooTest)
 expected = "* FooTest.test_dummy_attrs2 ... [ok]\n"
 do_test_with(desc, script, expected)
 
+### dummy_io (with with-statement)
+desc = "dummy_io (with with-statement)"
+script = r"""
+from __future__ import with_statement
+import sys, os
+from oktest import *
+from oktest.helper import *
+class FooTest(object):
+    def test_dummy_io(self):
+        sin, sout, serr = sys.stdin, sys.stdout, sys.stderr
+        with dummy_io("SOS") as d_io:
+            ok (sys.stdin)  != sin
+            ok (sys.stdout) != sout
+            ok (sys.stderr) != serr
+            ok (sys.stdin.read()) == "SOS"
+            sys.stdout.write("Haruhi")
+            sys.stderr.write("Sasaki")
+        ok (sys.stdin).is_(sin)
+        ok (sys.stdout).is_(sout)
+        ok (sys.stderr).is_(serr)
+        ok (d_io.stdout) == "Haruhi"
+        ok (d_io.stderr) == "Sasaki"
+run(FooTest)
+"""
+expected = "* FooTest.test_dummy_io ... [ok]\n"
+if with_statement_supported:
+    do_test_with(desc, script, expected)
+
+### dummy_io (without with-statement)
+desc = "dummy_io (without with-statement)"
+script = r"""
+import sys, os
+from oktest import *
+from oktest.helper import *
+class FooTest(object):
+    def test_dummy_io(self):
+        sin, sout, serr = sys.stdin, sys.stdout, sys.stderr
+        def f(arg1, arg2):
+            ok (sys.stdin)  != sin
+            ok (sys.stdout) != sout
+            ok (sys.stderr) != serr
+            ok (sys.stdin.read()) == "SOS"
+            sys.stdout.write(arg1)
+            sys.stderr.write(arg2)
+        d_io = dummy_io("SOS", f, "Haruhi", "Sasaki")
+        ok (sys.stdin).is_(sin)
+        ok (sys.stdout).is_(sout)
+        ok (sys.stderr).is_(serr)
+        ok (d_io.stdout) == "Haruhi"
+        ok (d_io.stderr) == "Sasaki"
+run(FooTest)
+"""
+expected = "* FooTest.test_dummy_io ... [ok]\n"
+if with_statement_supported:
+    do_test_with(desc, script, expected)
+
 ### chdir (with with-statement)
 desc = "chdir (with with-statement)"
 script = r"""
 from __future__ import with_statement
 import os
 from oktest import *
+from oktest.helper import *
 class FooTest(object):
     def test_chdir(self):
         with dummy_dir('_dummy_.d'):
@@ -909,6 +1010,7 @@ desc = "chdir (without with-statement)"
 script = r"""
 import os
 from oktest import *
+from oktest.helper import *
 class FooTest(object):
     def test_chdir2(self):
         def f():
@@ -930,6 +1032,7 @@ desc = "spec (with with-statement)"
 script = r"""
 from __future__ import with_statement
 from oktest import *
+from oktest.helper import *
 class FooTest(object):
     def test_spec1(self):
         with spec('1+1 is 2') as sp:
@@ -943,16 +1046,41 @@ run(FooTest)
 expected = """
 * FooTest.test_spec1 ... [ok]
 * FooTest.test_spec2 ... [NG] 0 == -1 : failed.
-   _test_.py:11: ok (1-1) == -1
+   _test_.py:12: ok (1-1) == -1
 """[1:]
 if with_statement_supported:
     do_test_with(desc, script, expected)
 
+### spec (with for-statement)
+desc = "spec (with for-statement)"
+script = r"""
+from __future__ import with_statement
+from oktest import *
+from oktest.helper import *
+class FooTest(object):
+    def test_spec1(self):
+        for sp in spec('1+1 is 2'):
+            ok (1+1) == 2
+        ok (sp.desc) == '1+1 is 2'
+    def test_spec2(self):
+        for _ in spec('1-1 is 0'):
+            ok (1-1) == -1
+            None.foobar   # raise NameError
+run(FooTest)
+"""
+expected = """
+* FooTest.test_spec1 ... [ok]
+* FooTest.test_spec2 ... [NG] 0 == -1 : failed.
+   _test_.py:12: ok (1-1) == -1
+"""[1:]
+do_test_with(desc, script, expected)
+
 ### using (with with-statement)
 desc = "using (with with-statement)"
 script = r"""
 from __future__ import with_statement
 from oktest import *
+from oktest.helper import *
 class FooTest(object):
     pass
 with using(FooTest):
@@ -970,10 +1098,315 @@ if with_statement_supported:
     do_test_with(desc, script, expected)
 
 
+### Tracer (function)
+desc = "Tracer (function)"
+script = r"""
+from oktest import *
+from oktest.tracer import Tracer
+def f1(a, b):
+    return f2(a + f3(b))
+def f2(a):
+    return a+2
+def f3(b):
+    return b*2
+tr = Tracer()
+f1 = tr.trace_func(f1)
+f2 = tr.trace_func(f2)
+f3 = tr.trace(f3)
+print(f1(3, 5))
+for call in tr:
+    print("---")
+    print(repr(call))
+    print(repr(call.receiver))
+    print(repr(call.name))
+    print(repr(call.args))
+    print(repr(call.kwargs))
+    print(repr(call.ret))
+"""[1:]
+expected = """
+15
+---
+f1(3, 5) #=> 15
+None
+'f1'
+(3, 5)
+{}
+15
+---
+f3(5) #=> 10
+None
+'f3'
+(5,)
+{}
+10
+---
+f2(13) #=> 15
+None
+'f2'
+(13,)
+{}
+15
+"""[1:]
+do_test_with(desc, script, expected)
+
+### Tracer (instance method)
+desc = "Tracer (instance method)"
+script = r"""
+from oktest import *
+from oktest.tracer import Tracer
+class Dummy(object):
+    def f1(self, x, y):
+        return [self.f2(x, y=y),
+                self.f2(y, y=x)]
+    def f2(self, x=None, y=None):
+        return x-y
+obj = Dummy()
+tr = Tracer()
+tr.trace_method(obj, 'f1', 'f2')
+ret = obj.f1(5, 3)
+print(ret)
+for call in tr:
+    print('---')
+    print(repr(call))
+    print(call.receiver is obj)
+    print(repr(call.name))
+    print(repr(call.args))
+    print(repr(call.kwargs))
+    print(repr(call.ret))
+"""[1:]
+expected = """
+[2, -2]
+---
+f1(5, 3) #=> [2, -2]
+True
+'f1'
+(5, 3)
+{}
+[2, -2]
+---
+f2(5, y=3) #=> 2
+True
+'f2'
+(5,)
+{'y': 3}
+2
+---
+f2(3, y=5) #=> -2
+True
+'f2'
+(3,)
+{'y': 5}
+-2
+"""[1:]
+do_test_with(desc, script, expected)
+
+### Tracer (class method)
+desc = "Tracer (class method)"
+script = r"""
+from oktest import *
+from oktest.tracer import Tracer
+class Dummy(object):
+    @classmethod
+    def f1(cls, x, y):
+        return [cls.__name__,
+                cls.f2(x, y=y),
+                cls.f2(y, y=x)]
+    @classmethod
+    def f2(cls, x=None, y=None):
+        return x-y
+tr = Tracer()
+tr.trace_method(Dummy, 'f1', 'f2')
+ret = Dummy.f1(5, 3)
+print(ret)
+for call in tr:
+    print("---")
+    print(repr(call))
+    print(call.receiver is Dummy)
+    print(repr(call.name))
+    print(repr(call.args))
+    print(repr(call.kwargs))
+    print(repr(call.ret))
+"""[1:]
+expected = """
+['Dummy', 2, -2]
+---
+f1(5, 3) #=> ['Dummy', 2, -2]
+True
+'f1'
+(5, 3)
+{}
+['Dummy', 2, -2]
+---
+f2(5, y=3) #=> 2
+True
+'f2'
+(5,)
+{'y': 3}
+2
+---
+f2(3, y=5) #=> -2
+True
+'f2'
+(3,)
+{'y': 5}
+-2
+"""[1:]
+do_test_with(desc, script, expected)
+
+### Tracer (fake)
+desc = "Tracer (fake_func)"
+script = r"""
+from oktest import *
+from oktest.tracer import Tracer
+def f(x, y, z=0):
+    return x + y + z
+def block(orig, *args, **kwargs):
+    v = orig(*args, **kwargs)
+    return 'v=%s' % v
+tr = Tracer()
+f = tr.fake_func(f, block)
+print(f(10, 20, z=7))  #=> 'v=37'
+print(repr(tr[0]))   #=> f(10, 20, z=7) #=> 'v=37'
+print(tr[0].receiver is None)  #=> True
+print(tr[0].name)    #=> f
+print(tr[0].args)    #=> (10, 20)
+print(tr[0].kwargs)  #=> {'z': 7}
+print(tr[0].ret)     #=> 'v=37'
+print('---')
+class Hello(object):
+    def hello(self, name):
+        return 'Hello %s!' % name
+    def hi(self):
+        pass
+obj = Hello()
+tr.fake_method(obj, hello=block, hi="Hi!", ya="Ya!")
+print(obj.hello('World'))  #=> v=Hello World!
+print(repr(tr[1]))    #=> hello('World') #=> 'v=Hello World!'
+print(tr[1].receiver is obj)  #=> True
+print(tr[1].name)     #=> hello
+print(tr[1].args)     #=> ('World',)
+print(tr[1].kwargs)   #=> {}
+print(tr[1].ret)      #=> v=Hello World!
+print('---')
+print(obj.hi('SOS'))  #=> Hi!
+print(repr(tr[2]))    #=> hi('SOS') #=> 'Hi!'
+print(tr[2].receiver is obj)  #=> True
+print(tr[2].name)     #=> hi
+print(tr[2].args)     #=> ('SOS',)
+print(tr[2].kwargs)   #=> {}
+print(tr[2].ret)      #=> Hi!
+print('---')
+print(obj.ya('SOS'))  #=> Ya!
+print(repr(tr[3]))    #=> ya('SOS') #=> 'Ya!'
+print(tr[3].receiver is obj)  #=> True
+print(tr[3].name)     #=> ya
+print(tr[3].args)     #=> ('SOS',)
+print(tr[3].kwargs)   #=> {}
+print(tr[3].ret)      #=> Ya!
+"""[1:]
+expected = """
+v=37
+f(10, 20, z=7) #=> 'v=37'
+True
+f
+(10, 20)
+{'z': 7}
+v=37
+---
+v=Hello World!
+hello('World') #=> 'v=Hello World!'
+True
+hello
+('World',)
+{}
+v=Hello World!
+---
+Hi!
+hi('SOS') #=> 'Hi!'
+True
+hi
+('SOS',)
+{}
+Hi!
+---
+Ya!
+ya('SOS') #=> 'Ya!'
+True
+ya
+('SOS',)
+{}
+Ya!
+"""[1:]
+do_test_with(desc, script, expected)
+
+
+### FakeObject class
+desc = "FakeObject class"
+script = r"""
+from oktest import *
+from oktest.tracer import FakeObject
+class FakeObjectTest(object):
+    def test_fake_object(self):
+        obj = FakeObject(hi="Hi", hello=lambda self, x: "Hello %s!" % x)
+        ok (obj.hi()) == 'Hi'
+        ok (obj.hello("SOS")) == 'Hello SOS!'
+        ok (obj._calls[0].name  ) == 'hi'
+        ok (obj._calls[0].args  ) == ()
+        ok (obj._calls[0].kwargs) == {}
+        ok (obj._calls[0].ret   ) == 'Hi'
+        ok (obj._calls[1].name  ) == 'hello'
+        ok (obj._calls[1].args  ) == ('SOS', )
+        ok (obj._calls[1].kwargs) == {}
+        ok (obj._calls[1].ret   ) == 'Hello SOS!'
+run()
+"""[1:]
+expected = """
+* FakeObjectTest.test_fake_object ... [ok]
+"""[1:]
+do_test_with(desc, script, expected)
+
+### Tracer.fake_obj()
+desc = "Tracer.fake_obj()"
+script = r"""
+import sys
+from oktest import *
+from oktest.tracer import Tracer
+class FakeObjectTest(object):
+    def test_dummy(self):
+        tr = Tracer()
+        ## create dummy object
+        obj1 = tr.fake_obj(hi="Hi!")
+        obj2 = tr.fake_obj(hello=lambda self, x: "Hello %s!" % x)
+        ## call dummy method
+        ok (obj2.hello("SOS")) == 'Hello SOS!'
+        ok (obj1.hi())         == 'Hi!'
+        ## check result
+        ok (tr[0].name  ) == 'hello'
+        ok (tr[0].args  ) == ('SOS', )
+        ok (tr[0].kwargs) == {}
+        ok (tr[0].ret   ) == 'Hello SOS!'
+        ok (tr[1].name  ) == 'hi'
+        ok (tr[1].args  ) == ()
+        ok (tr[1].kwargs) == {}
+        ok (tr[1].ret   ) == 'Hi!'
+        ## __iter__() and __eq__()
+        ok (tr[0].list())  == [obj2, 'hello', ('SOS',), {}, 'Hello SOS!']
+        ok (tr[0])         == [obj2, 'hello', ('SOS',), {}, 'Hello SOS!']
+        ok (tr[1].tuple()) == (obj1, 'hi', (), {}, 'Hi!')
+        ok (tr[1])         == (obj1, 'hi', (), {}, 'Hi!')
+run()
+"""[1:]
+expected = """
+* FakeObjectTest.test_dummy ... [ok]
+"""[1:]
+do_test_with(desc, script, expected)
+
+
 ## flatten
 desc = "flatten()"
 script = r"""
-from oktest import ok, run, flatten
+from oktest import ok, run
+from oktest.helper import flatten
 class FooTest(object):
     def test_flatten(self):
         ok (flatten([1, [2, 3, [4, 5, [[[6]]]], [7, 8]]])) == [1,2,3,4,5,6,7,8]
@@ -988,7 +1421,8 @@ do_test_with(desc, script, expected)
 desc = "rm_rf()"
 script = r"""
 import os
-from oktest import ok, not_ok, run, flatten, rm_rf
+from oktest import ok, not_ok, run
+from oktest.helper import flatten, rm_rf
 class FooTest(object):
     def setup(self):
         os.mkdir('_rm_rf')
@@ -1189,6 +1623,8 @@ ERROR: FooTest#test_error()
   File "_test_.py", line 11, in test_error
     ok (int('aaa')) == 0
 """[1:]
+if python2 and sys.version_info[1] <= 4:
+    expected = expected.replace("int() with base 10: 'aaa'", 'int(): aaa')
 os.environ['OKTEST_REPORTER'] = 'SimpleReporter'
 do_test_with(desc, script, expected)
 del os.environ['OKTEST_REPORTER']
